﻿Links = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceHook-2.1", "AceDB-2.0")

local L = AceLibrary("AceLocale-2.2"):new("Links")
local dewdrop = AceLibrary("Dewdrop-2.0")
local event

L:RegisterTranslations("enUS", function() return {
	["Link in chat"] = true,
	["Dressup"] = true,
	["Close all Tooltips"] = true,
	["Links - plenty of them"] = true,
	["modinvert"] = true,
	["Invert the alt-clic modifier behavior of Links"] = true,
	["autohide"] = true,
	["Automatically hide tooltip on escape"] = true,
	["CHAT_COMMANDS"] = { "/links" },
} end)

L:RegisterTranslations("koKR", function() return {
	["Link in chat"] = "대화창에 링크",
	["Dressup"] = "미리보기",
	["Close all Tooltips"] = "모든 툴팁 닫기",
	["Links - plenty of them"] = "링크 - 많은 아이템 링크 확인",
	["modinvert"] = "모드 전환",
	["Invert the alt-clic modifier behavior of Links"] = "ALT+클릭으로 링크 확인을 하도록 변경합니다.",
	["autohide"] = "자동 숨김",
	["Automatically hide tooltip on escape"] = "Esc키 입력시 툴팁을 숨김니다.",
} end)

L:RegisterTranslations("frFR", function() return {
	["Link in chat"] = "Ins\195\169rer un lien dans la fen\195\170tre de discussion",
	["Dressup"] = "Voir l'objet dans la cabine d'essayage",
	["Close all Tooltips"] = "Fermer toutes les fen\195\170tres d'objets",
	["Invert the alt-clic modifier behavior of Links"] = "Inverser le comportement de Links sur le alt-clic",
	["Automatically hide tooltip on escape"] = "Cacher automatiquement les tooltips lorsque Echap est appuy\195\169",
} end)

Links:RegisterDB("LinksDB")
Links:RegisterDefaults("profile", { 
	modinvert = false,
	autohide = true,
})

function Links:OnEnable()
	self.frames = {}
	self.index = 1
	
	self.options = {
		desc = L["Links - plenty of them"],
		type = "group",
		args = {
			modinvert = {
				name = L["modinvert"],
				desc = L["Invert the alt-clic modifier behavior of Links"],
				type = "toggle",
				get = function() return self.db.profile.modinvert end,
				set = function(value) self.db.profile.modinvert = value end,
			},
			autohide = {
				name = L["autohide"],
				desc = L["Automatically hide tooltip on escape"],
				type = "toggle",
				get = function() return self.db.profile.autohide end,
				set = function(value) self:SetAutoHide(value) end,
			}
		},
	}
	
	self:RegisterChatCommand(L["CHAT_COMMANDS"], self.options)
	self:Hook("SetItemRef", true)
end

function Links:SetAutoHide(value)
	local UISpecialFrames = UISpecialFrames
	self.db.profile.autohide = value
	for frame in pairs(self.frames) do
		local name = frame:GetName()
		if value then
			UISpecialFrames[#UISpecialFrames + 1] = name
		else
			for i = 1, #UISpecialFrames do
				if name == UISpecialFrames[i] then
					table.remove(UISpecialFrames, i)
					break
				end
			end
		end
	end
end

function Links:OpenMenu(frame)
	local link = frame.link
	
	dewdrop:Open(frame,
		"point", function() return "TOPLEFT", "TOPRIGHT" end,
		"dontHook", true,
		"children", function (level, name)
			dewdrop:AddLine(
				"text", L["Link in chat"],
				"func", function ()
					local _, l = GetItemInfo(link) 
					if not ChatFrameEditBox:IsVisible() then
							ChatFrame_OpenChat (l)
					else
							ChatFrameEditBox:SetText(ChatFrameEditBox:GetText()..l)
					end
					dewdrop:Close()
				end)
			dewdrop:AddLine(
				"text", L["Dressup"],
				"func", function () 
					DressUpItemLink(link) 
					dewdrop:Close()
				end)
			dewdrop:AddLine(
				"text", L["Close all Tooltips"],
				"func", function () 
					Links:HideAllFrames() 
					dewdrop:Close()
				end)
		end)
end

function Links:HideAllFrames()
	for frame in pairs(self.frames) do
		if frame:IsVisible() then
			frame:Hide()
		end
	end
end


function Links:MakeFrame()
	local name = string.format("LinksTooltip%d", self.index)
	local frame = CreateFrame("GameTooltip", name, UIParent, "GameTooltipTemplate")
	self.index = self.index + 1
	self.frames[frame] = true
	
	local close = CreateFrame("Button", name.."Close", frame)
	frame.close = close
	
	local makeTexture = function (path, blend)
		local t = close:CreateTexture()
		t:SetAllPoints(close)
		t:SetTexture(path)
		if blend then
			t:SetBlendMode(blend)
		end
		return t
	end

	close:SetNormalTexture(makeTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up"))
	close:SetPushedTexture(makeTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down"))
	close:SetHighlightTexture(makeTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight", "ADD"))
	
	close:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	close:SetScript("OnClick", 
		function (self, button, down) 
			if button == "LeftButton" then 
				frame:Hide()
			else
				Links:OpenMenu(frame) 
			end 
		end)
	
	
	close:SetWidth(32)
	close:SetHeight(32)
	close:ClearAllPoints()
	close:SetPoint("TOPRIGHT", 1, 0)
	
	close:Show()
	
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:SetToplevel(true)
	frame:SetFrameStrata("HIGH")
	frame:SetPoint("CENTER")
	
	frame:SetPadding(16)
	frame:RegisterForDrag("LeftButton")
	frame:SetBackdropColor(0.09, 0.12, 0.09)
	
	frame:SetScript("OnDragStart",
		function(self)
			self:StartMoving() 
		end)
	frame:SetScript("OnDragStop",
		function(self) 
			self:StopMovingOrSizing()
			Links:ValidateFramePosition(self)
		end)
	frame:SetScript("OnHide",
		function(self)
			local left, top = self:GetLeft(), self:GetTop()
			self:ClearAllPoints()
			self:SetPoint("TOPLEFT", left, top-UIParent:GetHeight())
			frame:SetBackdropColor(0.09, 0.12, 0.09)
			self.link = nil
			Links:OnHideTooltip(self)
		end)
	
	if GetCursorInfo then
		frame:SetScript("OnReceiveDrag", 
			function(self)
				local type, id, link = GetCursorInfo()
				if type == "item" then
					Links:OpenLink(link, self)
					ClearCursor()
				end
			end)
	end
	
	if self.db.profile.autohide then
		local UISpecialFrames = UISpecialFrames
		UISpecialFrames[#UISpecialFrames + 1] = name
	end
	
	local Mendeleev = Mendeleev
	if Mendeleev and Mendeleev.OnTooltipSetItem then
		Mendeleev:HookScript(frame, "OnTooltipSetItem")
	end
	
	return frame
end

do
    local ANCHORING_DISTANCE = 10
    local ANCHORING_OFFSET_X = 0
    local abs = math.abs
	
	local function GetAnchorX(frame, other)
        local fleft, fright = frame:GetLeft(), frame:GetRight()
        local oleft, oright = other:GetLeft(), other:GetRight()
		
		if abs(fleft - oleft) < ANCHORING_DISTANCE then
			return "LEFT", "LEFT"
		elseif abs(fleft - oright) < ANCHORING_DISTANCE then
			return "LEFT", "RIGHT"
		elseif abs(fright - oright) < ANCHORING_DISTANCE then
			return "RIGHT", "RIGHT"
		elseif abs(fright - oleft) < ANCHORING_DISTANCE then
			return "RIGHT", "LEFT"
		end
	end

	local function GetAnchorY(frame, other)
        local ftop, fbottom = frame:GetTop(), frame:GetBottom()
        local otop, obottom = other:GetTop(), other:GetBottom()
		
		if abs(ftop - otop) < ANCHORING_DISTANCE then
			return "TOP", "TOP"
		elseif abs(ftop - obottom) < ANCHORING_DISTANCE then
			return "TOP", "BOTTOM"
		elseif abs(fbottom - obottom) < ANCHORING_DISTANCE then
			return "BOTTOM", "BOTTOM"
		elseif abs(fbottom - otop) < ANCHORING_DISTANCE then
			return "BOTTOM", "TOP"
		end
	end

    function Links:IsAnchoring(frame, other)
		local fanchorx, oanchorx = GetAnchorX(frame, other)
		if not fanchorx then return end
		local fanchory, oanchory = GetAnchorY(frame, other)
		if not fanchory then return end
		
		if (fanchorx == oanchorx) == (fanchory == oanchory) then return end
		
		return fanchory..fanchorx, oanchory..oanchorx, 0, 0
    end
end

function Links:OnHideTooltip(frame)
    for other in pairs(self.frames) do
		if other.anchored == frame then
			local left, top = other:GetLeft(), other:GetTop()
			other:ClearAllPoints()
			other:SetPoint("TOPLEFT", left, top-UIParent:GetHeight())
			other:SetBackdropColor(0.09, 0.12, 0.09)
			other.anchored = nil
		end
	end
end

function Links:ValidateFramePosition(frame)
    -- check anchoring
    for other in pairs(self.frames) do
        if other ~= frame and other:IsVisible() and not other.anchored then
            local point, relative, x, y = self:IsAnchoring(frame, other)
            if point then
                frame:ClearAllPoints()
                frame:SetBackdropColor(0.18, 0.09, 0.09)
                frame:SetPoint(point, other, relative, x, y)
				frame.anchored = other
                return
            end
        end
    end
    local left, top = frame:GetLeft(), frame:GetTop()
    frame:ClearAllPoints()
    frame:SetPoint("TOPLEFT", left, top-UIParent:GetHeight())
    frame:SetBackdropColor(0.09, 0.12, 0.09)
	frame.anchored = nil
    ValidateFramePosition(frame)
end

function Links:GetAvailableFrame()
	for frame in pairs(self.frames) do
		if not frame:IsVisible() then
			return frame
		end
	end
	return self:MakeFrame()
end

function Links:OpenLink(link, frame)
	frame = frame or self:GetAvailableFrame()
	frame:ClearLines()
	frame:SetOwner(UIParent, "ANCHOR_PRESERVE")
	frame:SetHyperlink(link)
	frame.link = link
	if not event then
		event = AceLibrary("AceEvent-2.0")
	end
	event:TriggerEvent("Links_LinkOpened", link, frame)
	--[[
		The API is to do something like this in your addon :
		
		function MyAddon:OnEnable()
			...
			self:RegisterEvent("Links_LinkOpened")
		end
		
		function MyAddon:Links_LinkOpened(link, frame)
			...
		end
		
		Note: If you update Mendeleev, RatingBuster or PriceTag to use this API, then
		feel free to remove and commit the support code below.
	]]
	if Mendeleev and Mendeleev.ParseTooltip then Mendeleev:ParseTooltip(frame, link) end
	if RatingBuster then RatingBuster.ProcessTooltip(frame, link) end
	frame:Show()
end

function Links:SetItemRef(link, text, button)
	if string.sub(link, 1, 4) ~= "item" then
		return self.hooks.SetItemRef(link, text, button)
	else
		if not self.db.profile.modinvert then
			if IsModifierKeyDown() then
				return self.hooks.SetItemRef(link, text, button)
			end
		elseif not IsAltKeyDown() then
			return self.hooks.SetItemRef(link, text, button)
		end
	end
	for frame in pairs(self.frames) do
		if frame:IsVisible() and frame.link == link then
			frame:Hide()
			frame.link = nil
			return
		end
	end
	self:OpenLink(link)
end

